@@ -4,6 +4,7 @@
 from collections import Counter
 import numpy as np
 import xgi
+from xgi.exception import XGIError
 
 __all__ = [
     "degree_histogram",
@@ -57,19 +58,13 @@ def unique_edge_sizes(H, return_counts=False):
     ----------
     H : Hypergraph object
         The hypergraph of interest
-    return_counts : bool, default: False
-        Specifies whether to include the number of occurences of that edge size
 
     Returns
     -------
-    if return_counts:
-        numpy.ndarray, numpy.ndarray
-            Numpy arrays of the unique edge sizes and the number of each size respectively
-    else:
-        numpy.ndarray
-            A numpy array of the unique edge sizes
+    list()
+        The unique edge sizes
     """
-    return np.unique(list(H.edge_size), return_counts=return_counts)
+    return list({len(H.edges.members(edge)) for edge in H.edges})
 
 
 def frozen(*args, **kwargs):
@@ -77,7 +72,7 @@ def frozen(*args, **kwargs):
 
     Raises
     ------
-    xgi.HypergraphError
+    xgi.XGIError
         Raises error when user tries to modify the hypergraph
 
     Examples
@@ -87,9 +82,9 @@ def frozen(*args, **kwargs):
     >>> H = xgi.Hypergraph(hyperedge_list)
     >>> xgi.freeze(H)
     >>> H.add_node(5)
-    HypergraphError: "Frozen hypergraph can't be modified"
+    XGIError: "Frozen hypergraph can't be modified"
     """
-    raise xgi.HypergraphError("Frozen hypergraph can't be modified")
+    raise XGIError("Frozen hypergraph can't be modified")
 
 
 def freeze(H):
@@ -118,7 +113,7 @@ def freeze(H):
     >>> H = xgi.Hypergraph(hyperedge_list)
     >>> xgi.freeze(H)
     >>> H.add_node(5)
-    HypergraphError: "Frozen hypergraph can't be modified"
+    XGIError: "Frozen hypergraph can't be modified"
     """
     H.add_node = frozen
     H.add_nodes_from = frozen
@@ -200,11 +195,12 @@ def create_empty_copy(H, with_data=True):
     >>> H_copy.edges
     EdgeView([])
     """
-    H = H.__class__()
-    H.add_nodes_from(H.nodes(data=with_data))
+    H_copy = H.__class__()
+    H_copy.add_nodes_from(H.nodes)
     if with_data:
-        H._hypergraph.update(H._hypergraph)
-    return H
+        xgi.set_node_attributes(H_copy, dict(H._node_attr))
+        H_copy._hypergraph.update(H._hypergraph)
+    return H_copy
 
 
 def set_node_attributes(H, values, name=None):
@@ -255,7 +251,7 @@ def set_node_attributes(H, values, name=None):
         try:  # `values` is a dict
             for n, v in values.items():
                 try:
-                    H._node_attr[n][name] = values[n]
+                    H._node_attr[n][name] = v
                 except KeyError:
                     pass
         except AttributeError:  # `values` is a constant
@@ -366,7 +362,7 @@ def get_edge_attributes(H, name):
     get_node_attributes
     set_edge_attributes
     """
-    edge_data = H.edges.data
+    edge_data = H._edge_attr
     return {id: edge_data[edge][name] for edge in edge_data if name in edge_data[edge]}
 
 
