@@ -7,17 +7,11 @@ Multiedges and self-loops are allowed.
 """
 from copy import deepcopy
 
-from xgi.classes.reportviews import (
-    NodeView,
-    EdgeView,
-    DegreeView,
-    EdgeSizeView,
-)
-from xgi.exception import XGIError
-import xgi.convert as convert
-import numpy as np
-from xgi.utils import XGICounter
 import xgi
+import xgi.convert as convert
+from xgi.classes.reportviews import DegreeView, EdgeSizeView, EdgeView, NodeView
+from xgi.exception import XGIError
+from xgi.utils import XGICounter
 
 __all__ = ["Hypergraph"]
 
@@ -55,14 +49,7 @@ class Hypergraph:
         """
         self._edge_uid = XGICounter()
 
-        self.hypergraph_attr_dict_factory = self.hypergraph_attr_dict_factory
-        self.node_dict_factory = self.node_dict_factory
-        self.node_attr_dict_factory = self.node_attr_dict_factory
-        self.hyperedge_attr_dict_factory = self.hyperedge_attr_dict_factory
-
-        self._hypergraph = (
-            self.hypergraph_attr_dict_factory()
-        )  # dictionary for graph attributes
+        self._hypergraph = self.hypergraph_attr_dict_factory()
         self._node = self.node_dict_factory()  # empty node attribute dict
         self._node_attr = self.node_attr_dict_factory()
         self._edge = self.hyperedge_dict_factory()
@@ -73,32 +60,6 @@ class Hypergraph:
         # load hypergraph attributes (must be after convert)
         self._hypergraph.update(attr)
 
-    @property
-    def name(self):
-        """Get the string identifier of the hypergraph.
-
-        This hypergraph attribute appears in the attribute dict H._hypergraph
-        keyed by the string `"name"`. as well as an attribute (technically
-        a property) `H.name`. This is entirely user controlled.
-
-        Returns
-        -------
-        string
-            The name of the hypergraph
-        """
-        return self._hypergraph.get("name", "")
-
-    @name.setter
-    def name(self, s):
-        """Set the name of the hypergraph
-
-        Parameters
-        ----------
-        s : string
-            The desired name of the hypergraph.
-        """
-        self._hypergraph["name"] = s
-
     def __str__(self):
         """Returns a short summary of the hypergraph.
 
@@ -115,13 +76,10 @@ class Hypergraph:
         "Hypergraph named 'foo' with 0 nodes and 0 edges"
 
         """
-        return "".join(
-            [
-                type(self).__name__,
-                f" named {self.name!r}",
-                f" with {self.number_of_nodes()} nodes and {self.number_of_edges()} hyperedges",
-            ]
-        )
+        try:
+            return f"{type(self).__name__} named {self['name']} with {self.number_of_nodes()} nodes and {self.number_of_edges()} hyperedges"
+        except:
+            return f"Unnamed {type(self).__name__} with {self.number_of_nodes()} nodes and {self.number_of_edges()} hyperedges"
 
     def __iter__(self):
         """Iterate over the nodes. Use: 'for n in H'.
@@ -166,19 +124,16 @@ class Hypergraph:
         """
         return len(self._node)
 
-    def __getitem__(self, n):
-        """Returns a list of neighboring edge IDs of node n.  Use: 'H[n]'.
+    def __getitem__(self, attr):
+        """Read hypergraph attribute."""
+        try:
+            return self._hypergraph[attr]
+        except KeyError:
+            raise XGIError("This attribute has not been set.")
 
-        Returns
-        -------
-        list
-           A list of the edges of which the specified node is a part.
-
-        Notes
-        -----
-        H[n] is the same as H.nodes[n]
-        """
-        return self._node[n]
+    def __setitem__(self, attr, val):
+        """Write hypergraph attribute."""
+        self._hypergraph[attr] = val
 
     @property
     def shape(self):
@@ -246,8 +201,7 @@ class Hypergraph:
                 raise ValueError("None cannot be a node")
             self._node[node_for_adding] = list()
             self._node_attr[node_for_adding] = self.node_attr_dict_factory()
-        else:  # update attr even if node already exists
-            self._node_attr[node_for_adding].update(attr)
+        self._node_attr[node_for_adding].update(attr)
 
     def add_nodes_from(self, nodes_for_adding, **attr):
         """Add multiple nodes.
@@ -305,7 +259,7 @@ class Hypergraph:
         remove_nodes_from
         """
         try:
-            edge_neighbors = self.nodes[n]  # list handles self-loops (allows mutation)
+            edge_neighbors = self._node[n]  # list handles self-loops (allows mutation)
             del self._node[n]
             del self._node_attr[n]
         except KeyError as e:  # XGIError if n not in self
@@ -332,9 +286,8 @@ class Hypergraph:
         """
         for n in nodes:
             try:
-                edge_neighbors = self.nodes[
-                    n
-                ]  # list handles self-loops (allows mutation)
+                edge_neighbors = self._node[n]
+                # list handles self-loops (allows mutation)
                 del self._node[n]
                 del self._node_attr[n]
                 for edge in edge_neighbors:
@@ -351,21 +304,14 @@ class Hypergraph:
         """A NodeView of the Hypergraph as H.nodes or H.nodes().
 
         Can be used as `H.nodes` for data lookup and for set-like operations.
-        Can also be used as `H.nodes(data='color', default=None)` to return a
-        NodeDataView which reports specific node data but no set operations.
-        It presents a dict-like interface as well with `H.nodes.items()`
-        iterating over `(node, edge_ids)` 2-tuples. In addition,
-        a view `H.nodes.data('foo')` provides a dict-like interface to the
-        `foo` attribute of each node. `H.nodes.data('foo', default=1)`
-        provides a default for nodes that do not have attribute `foo`.
+        Can also be used as `H.nodes[id]` to return a
+        dictionary of the node attributes.
 
         Returns
         -------
         NodeView
             Allows set-like operations over the nodes as well as node
-            attribute dict lookup and calling to get a NodeDataView.
-            A NodeDataView iterates over `(n, data)` and has no set operations.
-            A NodeView iterates over `n` and includes set operations.
+            attribute dict lookup.
 
             When called, if data is False, an iterator over nodes.
             Otherwise an iterator of 2-tuples (node, attribute value)
@@ -461,19 +407,17 @@ class Hypergraph:
             return False
 
     def has_edge(self, edge):
-        """Specifies whether an edge appears in the hypergraph as a different ID.
+        """Whether an edge appears in the hypergraph.
 
         Parameters
         ----------
-        H : Hypergraph object
-            The hypergraph of interest
         edge : list or set
             A container of hashables that specifies an edge
 
         Returns
         -------
         bool
-           Whether or not the set appears as an edge in the hypergraph.
+           Whether or not edge is as an edge in the hypergraph.
 
         Examples
         --------
@@ -485,7 +429,7 @@ class Hypergraph:
         >>> H.has_edge({1, 3})
         False
         """
-        return set(edge) in [set(self.edges(e)) for e in self.edges]
+        return set(edge) in (set(self.edges(e)) for e in self.edges)
 
     def add_edge(self, edge, **attr):
         """Add an edge to the hypergraph. The universal ID
@@ -655,7 +599,7 @@ class Hypergraph:
         remove_edges_from : remove a collection of edges
         """
         try:
-            for node in self.edges[id]:
+            for node in self.edges.members(id):
                 self._node[node].remove(id)
             del self._edge[id]
             del self._edge_attr[id]
@@ -681,7 +625,7 @@ class Hypergraph:
         """
         for id in ebunch:
             try:
-                for node in self.edges[id]:
+                for node in self.edges.members(id):
                     self._node[node].remove(id)
                 del self._edge[id]
                 del self._edge_attr[id]
@@ -770,14 +714,12 @@ class Hypergraph:
 
     @property
     def edges(self):
-        """An EdgeView of the Hyperraph as H.edges or H.edges().
+        """An EdgeView of the Hypergraph as H.edges or H.edges().
 
         edges(self, nbunch=None, data=False, default=None)
 
         The EdgeView provides set-like operations on the edge IDs
-        as well as edge attribute lookup. When called, it also provides
-        an EdgeDataView object which allows control of access to edge
-        attributes (but does not provide set-like operations).
+        as well as edge attribute lookup.
 
         Parameters
         ----------
@@ -793,7 +735,12 @@ class Hypergraph:
         -----
         Nodes in nbunch that are not in the hypergraph will be (quietly) ignored.
         """
-        return EdgeView(self)
+        edges = EdgeView(self)
+        # Lazy View creation: overload the (class) property on the instance
+        # Then future H.edges use the existing View
+        # setattr doesn't work because attribute already exists
+        self.__dict__["edges"] = edges
+        return edges
 
     def get_edge_data(self, id, default=None):
         """Returns the attribute dictionary associated with edge id.
@@ -814,7 +761,9 @@ class Hypergraph:
             The edge attribute dictionary.
         """
         try:
-            return self.edges.data(id, default=default)
+            # this may fail because the ID may not exist
+            # or the property doesn't exist.
+            return self.edges[id]
         except KeyError:
             return default
 
@@ -1151,3 +1100,127 @@ class Hypergraph:
 
             bunch = bunch_iter(nbunch, self._node)
         return bunch
+
+    def max_edge_order(self):
+        """Returns the maximum order of edges in the hypergraph.
+
+        Returns
+        -------
+        d_max : int
+            Maximum order of edges in hypergraph
+        """
+
+        try:
+            edges = list(self._edge.values())
+            d_max = max([len(edge) for edge in edges]) - 1
+        except ValueError:  # if edges is empty
+            if len(self._node) > 0:
+                d_max = 0
+            else:
+                d_max = None
+
+        return d_max
+
+    def is_possible_order(self, d):
+        """Returns True if 'd' is a possible edge order.
+
+        Parameters
+        ----------
+        d : int
+            Order for which to check
+
+        Returns
+        -------
+        bool
+        """
+        d_max = self.max_edge_order()
+        return (d >= 1) and (d <= d_max)
+
+    def singleton_edges(self):
+        """Returns a dict of single edges"""
+
+        return {
+            id_: self._edge[id_]
+            for id_, size in dict(self.edge_size).items()
+            if size == 1
+        }
+
+    def remove_singleton_edges(self):
+        """Removes all singletons edges from the hypergraph"""
+
+        singleton_ids = [id_ for id_, size in dict(self.edge_size).items() if size == 1]
+        self.remove_edges_from(singleton_ids)
+        return None
+
+    def isolates(self, ignore_singletons=True):
+        """A list of all nodes disconnected from the hypergraph.
+
+        When ignore_singletons is True (default), a node is considered isolated frmo the
+        rest of the hypergraph when it is included in no edges of size two or more.  In
+        particular, whether the node is part of any singleton edges is irrelevant to
+        determine whether it is isolated.
+
+        When ignore_singletons is False, a node is isolated only when it is a member of
+        exactly zero edges, including singletons.
+
+        Parameters
+        ----------
+        ignore_singletons : bool, default False
+            Whether to consider singleton edges when searching for isolated nodes.
+
+        """
+        nodes_in_edges = set()
+        for idx in self.edges:
+            edge = self.edges.members(idx)
+            if ignore_singletons and len(edge) == 1:
+                continue
+            nodes_in_edges = nodes_in_edges.union(edge)
+        return set(self.nodes) - nodes_in_edges
+
+    def remove_isolates(self, ignore_singletons=True):
+        """Remove all nodes disconnected from the hypergraph.
+
+        Parameters
+        ----------
+        ignore_singletons : bool, default False
+            Whether to consider singleton edges when searching for isolated nodes.
+
+        See Also
+        --------
+        isolates
+
+        """
+        self.remove_nodes_from(self.isolates(ignore_singletons))
+        return self
+
+    def is_uniform(self):
+        """Returns d>=1 if the hypergraph is d-uniform, that is if
+        all edges in the hypergraph (excluding singletons, i.e. nodes)
+        have the same degree d. Returns d=None if not uniform.
+
+        This function can be used as a boolean check:
+        >>> if H.is_uniform()
+        works as expected.
+
+        Returns:
+        --------
+            uniform : bool
+                True if the hypergraph is d-uniform. None if the
+                hypergraph has no edges (other than singleton edges).
+            d : int (optional return)
+        """
+
+        edge_sizes = set(dict(self.edge_size()).values())
+        if 1 in edge_sizes:
+            edge_sizes.remove(1)  # discard singleton edges
+
+        if len(edge_sizes) == 0:  # no edges
+            d = False  # not uniform
+        else:
+            uniform = len(edge_sizes) == 1
+            if uniform:
+                d = list(edge_sizes)[0] - 1  # order of all edges
+            else:
+                d = False
+
+        return d
