@@ -24,11 +24,11 @@ def generate_bipartite_edgelist(H, delimiter=" "):
         Each entry is a line to be written to the output file.
     """
     for id in H.edges:
-        for node in H.edges[id]:
+        for node in H.edges.members(id):
             yield delimiter.join(map(str, [node, id]))
 
 
-def write_bipartite_edgelist(H, path, delimiter=" ", data=True, encoding="utf-8"):
+def write_bipartite_edgelist(H, path, delimiter=" ", encoding="utf-8"):
     """Write a Hypergraph object to a file
     as a bipartite edgelist.
 
@@ -40,8 +40,6 @@ def write_bipartite_edgelist(H, path, delimiter=" ", data=True, encoding="utf-8"
         The path of the file to write to
     delimiter: char, default: space (" ")
         Specifies the delimiter between hyperedge members
-    data: bool, default: True
-        Specifies whether to output the edge attributes
     encoding: string, default: "utf-8"
         Encoding of the file
 
@@ -70,7 +68,7 @@ def read_bipartite_edgelist(
     delimiter=None,
     create_using=None,
     nodetype=None,
-    data=False,
+    dual=False,
     encoding="utf-8",
 ):
     """Read a file containing a bipartite edge list and
@@ -88,8 +86,9 @@ def read_bipartite_edgelist(
         The hypergraph object to add the data to, by default None
     nodetype: type
         type that the node labels will be cast to
-    data: bool, default: False
-        Specifies whether there is a dictionary of data at the end of the line.
+    dual: bool, default: False
+        Specifies whether the node IDs are in the second column. If False,
+        the node IDs are in the first column.
     encoding: string, default: "utf-8"
         Encoding of the file
 
@@ -117,36 +116,53 @@ def read_bipartite_edgelist(
             delimiter=delimiter,
             create_using=create_using,
             nodetype=nodetype,
+            dual=dual,
         )
 
 
 def parse_bipartite_edgelist(
-    lines, comments="#", delimiter=None, create_using=None, nodetype=None
+    lines, comments="#", delimiter=None, create_using=None, nodetype=None, dual=False
 ):
     """
     A helper function to read a iterable of strings containing a bipartite edge list and
     convert it to a Hypergraph object.
 
+    Reads the first two entries of each line and assumes that the first entry is a node
+    ID and that the second entry is an edge ID. Raises error if there are fewer than two
+    entries.
+
     Parameters
     ----------
     lines: iterable of strings
-        Lines where each line is an edge
+        Lines where each line is a bipartite edge
     comments: string, default: "#"
         The token that denotes comments to ignore
     delimiter: char, default: space (" ")
         Specifies the delimiter between hyperedge members
-    create_using:
+    create_using : Hypergraph constructor, optional
+        The hypergraph object to add the data to, by default None
     nodetype: type
         type that the node labels will be cast to
     data: bool, default: False
         Specifies whether there is a dictionary of data at the end of the line.
 
+    Raises
+    ------
+    XGIError
+        If a line contains fewer than two entries
+    TypeError
+        If node types fail to be converted
+
     Returns
     -------
     A Hypergraph object
         The loaded hypergraph
     """
     H = xgi.empty_hypergraph(create_using)
+
+    node_index = 1 if dual else 0
+    edge_index = 0 if dual else 1
+
     for line in lines:
         if comments is not None:
             p = line.find(comments)
@@ -155,15 +171,15 @@ def parse_bipartite_edgelist(
             if not line:
                 continue
         s = line.strip().split(delimiter)
-        if len(s) != 2:
-            raise XGIError("Each line must contain two entries!")
+        if len(s) < 2:
+            raise XGIError("Each line must contain at least two entries!")
         # no data or data type specified
 
         if nodetype is not None:
             try:
-                H.add_node_to_edge(s[1], nodetype(s[0]))
+                H.add_node_to_edge(s[edge_index], nodetype(s[node_index]))
             except Exception as e:
                 raise TypeError(f"Failed to convert nodes to type {nodetype}.") from e
         else:
-            H.add_node_to_edge(s[1], s[0])
+            H.add_node_to_edge(s[edge_index], s[node_index])
     return H
